/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors. 
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.sts.auth;

import java.util.HashMap;

import javax.security.auth.login.AppConfigurationEntry;
import javax.security.auth.login.AppConfigurationEntry.LoginModuleControlFlag;

import weblogic.logging.NonCatalogLogger;
import weblogic.management.security.ProviderMBean;
import weblogic.security.provider.PrincipalValidatorImpl;
import weblogic.security.spi.AuthenticationProviderV2;
import weblogic.security.spi.IdentityAsserterV2;
import weblogic.security.spi.PrincipalValidator;
import weblogic.security.spi.SecurityServices;

/**
 * A Weblogic Authentication Provider that uses SAML2 tokens to authenticate in PicketLink STS.
 * This authentication provider should be set as the first authenticator in the realm list and be marked
 * with the SUFFICIENT control flag.
 * 
 * @author <a href="mmoyses@redhat.com">Marcus Moyses</a>
 * @version $Revision: 1 $
 */
public class SimpleSAML2STSAuthenticationProviderImpl implements AuthenticationProviderV2
{
   /** description of the authentication provider */
   private String description;
   
   /** properties file for role mapping */
   private String rolesFile;

   /** properties file for STS configuration */
   private String stsConfigurationFile;
   
   /** login module control flag */
   private LoginModuleControlFlag controlFlag;
   
   /** logger */
   NonCatalogLogger logger = new NonCatalogLogger("SimpleSAML2STSAuthenticator");
   
   /**
    * @see AuthenticationProviderV2#getAssertionModuleConfiguration()
    */
   public AppConfigurationEntry getAssertionModuleConfiguration()
   {
      return null;
   }

   /**
    * @see AuthenticationProviderV2#getIdentityAsserter()
    */
   public IdentityAsserterV2 getIdentityAsserter()
   {
      return null;
   }

   /**
    * @see AuthenticationProviderV2#getLoginModuleConfiguration()
    */
   public AppConfigurationEntry getLoginModuleConfiguration()
   {
      HashMap options = new HashMap();
      options.put("rolesFile", rolesFile);
      options.put("stsConfigurationFile", stsConfigurationFile);
      return getConfiguration(options);
   }

   /**
    * @see AuthenticationProviderV2#getPrincipalValidator()
    */
   public PrincipalValidator getPrincipalValidator()
   {
      return new PrincipalValidatorImpl();
   }

   /**
    * @see AuthenticationProviderV2#getDescription()
    */
   public String getDescription()
   {
      return description;
   }

   /**
    * @see AuthenticationProviderV2#initialize(ProviderMBean, SecurityServices)
    */
   public void initialize(ProviderMBean provider, SecurityServices services)
   {
      description = provider.getDescription();
      SimpleSAML2STSAuthenticatorMBean myMBean = (SimpleSAML2STSAuthenticatorMBean) provider;
      String flag = myMBean.getControlFlag();
      if (flag.equalsIgnoreCase("REQUIRED"))
      {
         controlFlag = LoginModuleControlFlag.REQUIRED;
      }
      else if (flag.equalsIgnoreCase("OPTIONAL"))
      {
         controlFlag = LoginModuleControlFlag.OPTIONAL;
      }
      else if (flag.equalsIgnoreCase("REQUISITE"))
      {
         controlFlag = LoginModuleControlFlag.REQUISITE;
      }
      else if (flag.equalsIgnoreCase("SUFFICIENT"))
      {
         controlFlag = LoginModuleControlFlag.SUFFICIENT;
      }
      else
      {
         throw new IllegalArgumentException("Invalid flag value" + flag);
      }
      logger.debug("Control flag set to " + flag);
      rolesFile = myMBean.getRolesFile();
      logger.debug("RolesFile set to " + rolesFile);
      if (rolesFile == null || rolesFile.equals(""))
         throw new IllegalArgumentException("RolesFile must be set");
      stsConfigurationFile = myMBean.getSTSConfigurationFile();
      logger.debug("STSConfigurationFile set to " + stsConfigurationFile);
      if (stsConfigurationFile == null || stsConfigurationFile.equals(""))
         throw new IllegalArgumentException("STSConfigurationFile must be set");
   }

   /**
    * @see AuthenticationProviderV2#shutdown()
    */
   public void shutdown()
   {
   }

   private AppConfigurationEntry getConfiguration(HashMap options)
   {
      return new AppConfigurationEntry("org.jboss.sts.auth.SimpleSAML2STSLoginModuleImpl", controlFlag, options);
   }

}
