/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors. 
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.sts.auth;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;
import java.util.Properties;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;
import javax.xml.bind.JAXBElement;

import org.picketlink.identity.federation.core.exceptions.ProcessingException;
import org.picketlink.identity.federation.core.wstrust.STSClient;
import org.picketlink.identity.federation.core.wstrust.SamlCredential;
import org.picketlink.identity.federation.core.wstrust.WSTrustException;
import org.picketlink.identity.federation.core.wstrust.STSClientConfig.Builder;
import org.picketlink.identity.federation.core.wstrust.plugins.saml.SAMLUtil;
import org.picketlink.identity.federation.saml.v2.assertion.AssertionType;
import org.picketlink.identity.federation.saml.v2.assertion.NameIDType;
import org.picketlink.identity.federation.saml.v2.assertion.SubjectType;
import org.w3c.dom.Element;

import weblogic.logging.NonCatalogLogger;
import weblogic.security.principal.WLSGroupImpl;
import weblogic.security.principal.WLSUserImpl;

/**
 * A simple login module that checks if the password is a valid STS token in the String format.
 * If the token is valid, proceed to map the user's roles from a properties file.
 * If the token is invalid, let authentication proceed to the next authenticator.
 * 
 * @author <a href="mmoyses@redhat.com">Marcus Moyses</a>
 * @version $Revision: 1 $
 */
public class SimpleSAML2STSLoginModuleImpl implements LoginModule
{
   /** the subject */
   private Subject subject;

   /** callbackhandler passed to this login module */
   private CallbackHandler callbackHandler;

   /** flag to indicate is principals were added to the subject */
   private boolean principalsInSubject;

   /** flag to indicate if login was successful */
   private boolean loginSucceeded;

   /** principals to add to the subject */
   protected Collection principalsForSubject = new ArrayList();

   /** properties file for role mapping */
   private String rolesFile;

   /** STS configuration file */
   private String stsConfigurationFile;

   /** logger */
   NonCatalogLogger logger = new NonCatalogLogger("SimpleSAML2STSAuthenticator");

   /**
    * @see LoginModule#abort()
    */
   public boolean abort() throws LoginException
   {
      if (principalsInSubject)
      {
         // remove all principals from the subject
         subject.getPrincipals().removeAll(principalsForSubject);
         principalsInSubject = false;
      }
      return true;
   }

   /**
    * @see LoginModule#commit()
    */
   public boolean commit() throws LoginException
   {
      if (loginSucceeded)
      {
         // add all principals to the subject
         subject.getPrincipals().addAll(principalsForSubject);
         principalsInSubject = true;
         return true;
      }
      else
      {
         // let invocation go to the next authenticator
         return false;
      }
   }

   /**
    * @see LoginModule#initialize()
    */
   public void initialize(Subject subject, CallbackHandler callbackHandler, Map sharedState, Map options)
   {
      this.subject = subject;
      this.callbackHandler = callbackHandler;
      this.rolesFile = (String) options.get("rolesFile");
      this.stsConfigurationFile = (String) options.get("stsConfigurationFile");
   }

   /**
    * @see LoginModule#login()
    */
   public boolean login() throws LoginException
   {
      Callback[] callbacks = getCallbacks();
      PasswordCallback passwordCallback = (PasswordCallback) callbacks[0];
      char[] password = passwordCallback.getPassword();
      if (password == null)
         throw new LoginException("Credential can not be null");
      passwordCallback.clearPassword();
      String credential = new String(password);
      Element assertionElement = null;
      try
      {
         // check if credential is a token
         assertionElement = SamlCredential.assertionToElement(credential);
      }
      catch (ProcessingException pe)
      {
         // not a token
         logger.debug("Credential is not a SAML2 token. Assuming it is a password");
      }

      if (stsConfigurationFile == null || stsConfigurationFile.equals(""))
      {
         logger.error("STS configuration file must be set");
         throw new LoginException("STS configuration file must be set");
      }

      boolean isTokenValid = false;
      Builder builder = new Builder(stsConfigurationFile);
      STSClient client = new STSClient(builder.build());
      // only validate token if it is a token
      if (assertionElement != null)
      {
         try
         {
            isTokenValid = client.validateToken(assertionElement);
         }
         catch (WSTrustException we)
         {
            LoginException exception = new LoginException("Failed to validate assertion using STS: " + we.getMessage());
            exception.initCause(we);
            logger.error("Error validating token", we);
            throw exception;
         }
      }

      if (isTokenValid)
      {
         logger.debug("SAML2 token is valid");
         // if the assertion is valid, create a principal containing the assertion subject
         String userName = null;
         try
         {
            AssertionType assertion = SAMLUtil.fromElement(assertionElement);
            SubjectType subject = assertion.getSubject();
            if (subject != null)
            {
               for (JAXBElement<?> element : subject.getContent())
               {
                  if (element.getDeclaredType().equals(NameIDType.class))
                  {
                     NameIDType nameID = (NameIDType) element.getValue();
                     userName = nameID.getValue();
                     logger.debug("Assertion subject: " + userName);
                     // add the user to the principals
                     principalsForSubject.add(new WLSUserImpl(userName));
                     break;
                  }
               }
            }
         }
         catch (Exception e)
         {
            LoginException exception = new LoginException("Failed to parse assertion element" + e.getMessage());
            exception.initCause(e);
            logger.error("Failed to parse assertion element", e);
            throw exception;
         }
         loginSucceeded = true;
         // add the roles to the principals
         addGroups(userName);
         return true;
      }
      else
      {
         logger.debug("SAML2 token is invalid. Assuming it is a password");
         loginSucceeded = false;
         // let invocation go to the next authenticator
         return false;
      }
   }

   /**
    * @see LoginModule#logout()
    */
   public boolean logout() throws LoginException
   {
      return true;
   }

   private Callback[] getCallbacks() throws LoginException
   {
      if (callbackHandler == null)
      {
         throw new LoginException("No CallbackHandler Specified");
      }
      Callback[] callbacks = new Callback[1];
      callbacks[0] = new PasswordCallback("password: ", false);

      try
      {
         callbackHandler.handle(callbacks);
      }
      catch (IOException e)
      {
         logger.error("Error handling callbacks", e);
         throw new LoginException(e.toString());
      }
      catch (UnsupportedCallbackException e)
      {
         logger.error("Error handling callbacks", e);
         throw new LoginException(e.toString() + " " + e.getCallback().toString());
      }
      return callbacks;
   }

   /**
    * Adds the user's roles (groups) to the protected Collection principalsForSubject.
    * This method should be overridden by the subclasses
    * 
    * @param userName The name of the user whom the roles should be retrieved
    */
   protected void addGroups(String userName)
   {
      if (rolesFile != null && !rolesFile.equals(""))
      {
         try
         {
            InputStream is = new FileInputStream(rolesFile);
            Properties properties = new Properties();
            properties.load(is);
            is.close();
            String rolesString = properties.getProperty(userName);
            if (rolesString != null && !rolesString.equals(""))
            {
               String[] roles = rolesString.split(",");
               for (int i = 0; i < roles.length; i++)
               {
                  logger.debug("Adding role " + roles[i]);
                  principalsForSubject.add(new WLSGroupImpl(roles[i]));
               }
            }
         }
         catch (FileNotFoundException e)
         {
            logger.error("Error opening the roles file. No roles will be assigned", e);
         }
         catch (IOException e)
         {
            logger.error("Error while reading the roles file. No roles will be assigned", e);
         }
      }
      else
      {
         logger.error("Roles file must be set. No roles will be assigned");
      }
   }

}
